/* proc.h: MN2WS0050 processor description
 *
 * Copyright (C) 2002 Red Hat, Inc. All Rights Reserved.
 * Written by David Howells (dhowells@redhat.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#ifndef _ASM_PROC_PROC_H
#define _ASM_PROC_PROC_H

#define PROCESSOR_VENDOR_NAME	"Panasonic"
#define PROCESSOR_MODEL_NAME	"mn2ws0050"

#endif /* _ASM_PROC_PROC_H */
